let versionPressCount = 0;
let devTimer;
var silenceStatus = false;

window.fn = {};

window.fn.open = function () {
  var menu = document.getElementById("menu");
  if (isDeveloper()) {
    $("#devButton").show();
  }
  menu.open();
};

window.fn.load = function (page) {
  var content = document.getElementById("content");
  var menu = document.getElementById("menu");
  content.load(page).then(menu.close.bind(menu));
};

$(function () {
  if (!AppFunctions.hasOverlayPermission()) {
    AppFunctions.requestOverlayPermission();
  }
  if (!AppFunctions.hasLocationPermission()) {
    AppFunctions.requestLocationPermission();
  }
  ons.ready(function () {
    checkUpdates();
  });
});

function delay(t, v) {
  return new Promise((resolve) => setTimeout(resolve, t, v));
}

async function loadalHome() {
  $("#loadal_content").text("Loading...");
  $("#loadal").show();
}

async function updateHome() {
  loadalHome().then((response) => {
    $("#connected_wifi").text(AppFunctions.getCurrentWifi());
    $("#loadal").hide();
  });
}

async function updateAbout() {
  $("#version_name").text(AppFunctions.getAppVersionName());
  $("#version_code").text(AppFunctions.getAppVersionCode());
}

async function updateSettings() {
}

async function updateDev() {
    $("#dev_quiet").prop("checked", AppFunctions.getSetting("quiet_mode", "0") == "1");
    $("#dev_mode").prop("checked", AppFunctions.getSetting("developer", "0") == "1");
}

function toggleQuietMode(state) {
    if (state) {
        AppFunctions.setSetting("quiet_mode", "1");
    } else {
        AppFunctions.setSetting("quiet_mode", "0");
    }
}

function toggleDevMode(state) {
    if (state) {
      unlockDeveloper();
    } else {
      lockDeveloper();
    }
}

document.addEventListener("show", async function (event) {
  var page = event.target;

  if (page.id == "devices") {
    await updateDevices();
  } else if (page.id == "home") {
    await updateHome();
  } else if (page.id == "about") {
    await updateAbout();
  } else if (page.id == "settings") {
    await updateSettings();
  } else if (page.id == "dev") {
    if (isDeveloper()) {
      await updateDev();
    } else {
      fn.load("home.html");
    }
  }
});

$(document).on("changestate", "ons-pull-hook", function (event) {
  if (event.target.state == "action") {
    var page = event.target.parentElement.parentElement.id;
    fn.load(page + ".html");
  }
});

$(document).on("click", "#settings_list>.string_setting", function (event) {
  $("#setting_title").text(
    event.target.parentElement.parentElement.getAttribute("data-title")
  );
  $("#setting_key").val(
    event.target.parentElement.parentElement.getAttribute("data-key")
  );
  $("#setting_value").val(
    AppFunctions.getSetting(
      event.target.parentElement.parentElement.getAttribute("data-key")
    )
  );
  delay(200).then((response) => {
    $("#setting_edit").show();
  });
});

function isDeveloper() {
  return AppFunctions.getSetting("developer") == "1";
}

function unlockDeveloper() {
  AppFunctions.setSetting("developer", "1");
}

function lockDeveloper() {
  AppFunctions.setSetting("developer", "0");
}

function checkVersionPresses() {
  versionPressCount++;
  if (versionPressCount === 1) {
    devTimer = setTimeout(() => {
      versionPressCount = 0;
    }, 5000);
  }
  if (versionPressCount > 10) {
    clearTimeout(devTimer);
    ons.notification.toast("Unlocking developer settings", { timeout: 2500 });
    unlockDeveloper();
    versionPressCount = 0;
  }
}

$(document).on("click", ".version", function (event) {
  checkVersionPresses();
});

function processSettingEdit() {
  $("#setting_edit").hide();
  AppFunctions.setSetting($("#setting_key").val(), $("#setting_value").val());
  fn.load("settings.html");
}

function processJSON(json) {
    var content = JSON.parse(json);
    AppFunctions.setSetting("hostname", content.hostname);

    var registrationData = {
        "key": content.key,
    }

    var registrationPayload = JSON.stringify(registrationData);

    var responseJson = AppFunctions.registerDevice(registrationPayload);
    var response = JSON.parse(responseJson);

    console.log(response);

    if (response.success) {
        AppFunctions.setSetting("key", response.key);
        ons.notification.toast("Registered with server", { timeout: 2500 });
        fn.load("home.html");
    }
}

function startSettingsReset() {
  ons.notification
    .confirm({
      title: "Reset Settings",
      message: "Would you like to reset all settings to default values?",
      buttonLabels: ["Cancel", "Reset"],
    })
    .then((response) => {
      if (response == 1) {
        AppFunctions.setSetting("hostname", "");
        AppFunctions.setSetting("key", "");
        fn.load("settings.html");
      }
    });
}

function startAlarmAudio() {
    AppFunctions.startAlarmAudio();
}

function stopAlarmAudio() {
    AppFunctions.stopAlarmAudio();
}

function panicking(heartbeat) {
    console.log("Panicking");
    $(".panic").addClass("active");
    $(".panic").text("PANICKING!");
    $("#panicStatus").text("Active!");

    var heartbeatData = JSON.parse(heartbeat);
    console.log(heartbeat);

    $("#panicReason").text(heartbeatData.reason);
    $("#panicTriggeredBy").text(heartbeatData.triggered_by);
    $("#panicTriggeredAt").text(heartbeatData.triggered_at);

    if (!isAlarmModeActive()) {
        startAlarmAudio();
    }

    $('#overlay').fadeOut();
}

function notPanicking() {
    console.log("Not panicking");
    $(".panic").removeClass("active");
    $(".panic").text("PANIC!");
    $("#panicStatus").text("Inactive");
    $("#panicReason").text("None");
    $("#panicTriggeredBy").text("None");
    $("#panicTriggeredAt").text("None");

    if (isAlarmModeActive()) {
        stopAlarmAudio();
    }
}

function showOpenSourceLicenses() {
  $("#licenses").show();
}

var lastAction = Date.now();
var alarmMode = false;

$(document).on('mousemove keypress', function() {
    lastAction = Date.now();
    $('#overlay').fadeOut();
});

$('#overlay').on('click', function() {
    $(this).fadeOut();
});

function isAlarmModeActive() {
  return AppFunctions.isAlarmAudioPlaying();
}

var checkInterval = setInterval(function() {
  if (!isAlarmModeActive()) {
    silenceStatus = false;
    if (lastAction < Date.now() - 10000) {
        $('#overlay').fadeIn();
    }
  }
}, 1000);

function panicButtonPress() {
    if (isAlarmModeActive()) {
        ons.notification.confirm({
            title: 'Alarm',
            message: 'Would you like to silence the alarm for yourself or solve it for everyone?',
            buttonLabels: ['Solve', 'Silence', 'Cancel'],
            callback: function(idx) {
                switch(idx) {
                    case 0:
                        solveAlarm();
                        break;
                    case 1:
                        silenceAlarm();
                        break;
                    case 2:
                        break;
                }
            }
        });
    } else {
        ons.notification.confirm({
            title: 'Panic',
            message: 'Would you like to trigger a panic?',
            buttonLabels: ['Yes', 'No'],
            callback: function(idx) {
                switch(idx) {
                    case 0:
                        triggerPanic();
                        break;
                    case 1:
                        break;
                }
            }
        });
    }
}

function triggerPanic() {
    ons.notification.prompt({
        title: 'Panic',
        message: 'Please enter a reason for the panic',
        cancelable: true,
        callback: function(reason) {
            if (reason) {
                var panicData = {
                    "reason": reason,
                    "key": AppFunctions.getSetting("key")
                }
                var panicPayload = JSON.stringify(panicData);
                var responseJson = AppFunctions.triggerPanic(panicPayload);
                var response = JSON.parse(responseJson);
                if (response.success) {
                    ons.notification.toast("Panic triggered", { timeout: 2500 });
                } else {
                    ons.notification.toast("Failed to trigger panic", { timeout: 2500 });
                }
            }
        }
    });
}

function silenceAlarm() {
    AppFunctions.silenceAlarmAudio();
    silenceStatus = true;
}

function solveAlarm() {
    ons.notification.prompt({
        title: 'Solve Panic',
        message: 'Please enter a comment to explain how you solved the panic',
        cancelable: true,
        callback: function(comment) {
            if (comment) {
                var commentData = {
                    "reason": comment,
                    "key": AppFunctions.getSetting("key")
                }
                var commentPayload = JSON.stringify(commentData);
                var responseJson = AppFunctions.solvePanic(commentPayload);
                var response = JSON.parse(responseJson);
                if (response.success) {
                    ons.notification.toast("Panic solved", { timeout: 2500 });
                    silenceAlarmAudio();
                } else {
                    ons.notification.toast("Failed to solve panic", { timeout: 2500 });
                }
            }
        }
    });
}

function checkUpdates() {
  var dataJson = AppFunctions.getUpdates()
  var data = JSON.parse(dataJson);

  console.log(dataJson);

  var latestVersion = data.versions.android.build;
  var currentVersion = AppFunctions.getAppVersionCode();

  if (latestVersion > currentVersion) {
    ons.notification.confirm({
      title: 'Update Available',
      message: 'Would you like to update to the latest version?',
      buttonLabels: ['Yes', 'No'],
      callback: function(idx) {
        switch(idx) {
          case 0:
            AppFunctions.downloadUpdate(data.versions.android.url);
            break;
          case 1:
            break;
        }
      }
    });
  }
}

setInterval(checkUpdates, 1000 * 60 * 60 * 24);