package systems.kumi.expalert

import android.content.Context
import android.content.Intent
import android.content.Intent.FLAG_ACTIVITY_NEW_TASK
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.media.AudioManager
import android.media.MediaPlayer
import android.net.Uri
import android.net.wifi.WifiManager
import android.preference.PreferenceManager
import android.provider.Settings
import android.util.Log
import android.webkit.JavascriptInterface
import androidx.core.app.ActivityCompat.startActivityForResult
import androidx.core.content.ContextCompat.startActivity
import org.json.JSONObject


class AppFunctions(context: Context) {
    private var context: Context = context
    private var preferences: SharedPreferences = PreferenceManager.getDefaultSharedPreferences(context)!!
    private var connection: ServerConnection = ServerConnection(context)
    var mediaPlayer: MediaPlayer = MediaPlayer()

    fun mapToString(map: HashMap<String, String>): String {
        val jsonObject = JSONObject()
        for ((key, value) in map.iterator()) {
            jsonObject.put(key, value.trim('"'))
        }
        return jsonObject.toString()
    }

    @JavascriptInterface
    fun openWifiSettings() {
        startActivity(context, Intent("android.settings.WIFI_SETTINGS").setFlags(FLAG_ACTIVITY_NEW_TASK), null)
    }

    @JavascriptInterface
    fun getCurrentWifi(): String? {
        val wifiManager = context.getSystemService(Context.WIFI_SERVICE) as WifiManager
        val wifiInfo = wifiManager.connectionInfo
        val ssid = wifiInfo.ssid
        Log.d("WIFI", "SSID: $ssid")
        return ssid
    }

    @JavascriptInterface
    fun getSetting(key: String, default: String = ""): String? {
        var value = preferences.getString(key, default)
        return if (value.isNullOrBlank()) default else value
    }

    @JavascriptInterface
    fun getSetting(key: String): String? {
        return this.getSetting(key, "")
    }

    @JavascriptInterface
    fun setSetting(key: String, value: String) {
        Log.i("SETTING", "$key,$value")
        preferences.edit().putString(key, value).apply()
    }

    @JavascriptInterface
    fun getAppVersionName(): String {
        return BuildConfig.VERSION_NAME
    }

    @JavascriptInterface
    fun getAppVersionCode(): Int {
        return BuildConfig.VERSION_CODE
    }

    @JavascriptInterface
    fun openBrowser(url: String) {
        val i = Intent(Intent.ACTION_VIEW)
        i.data = Uri.parse(url)
        startActivity(context, i, null)
    }

    @JavascriptInterface
    fun requestLocationPermission() {
        val i = Intent("android.settings.APPLICATION_DETAILS_SETTINGS")
        i.data = Uri.fromParts("package", context.packageName, null)
        startActivity(context, i, null)
    }

    @JavascriptInterface
    fun hasLocationPermission(): Boolean {
        return context.checkSelfPermission(android.Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED
    }

    @JavascriptInterface
    fun requestOverlayPermission() {
        val intent = Intent(
            Settings.ACTION_MANAGE_OVERLAY_PERMISSION,
            Uri.parse("package:systems.kumi.expalert")
        )
        startActivity(context, intent, null)
    }

    @JavascriptInterface
    fun hasOverlayPermission(): Boolean {
        return Settings.canDrawOverlays(this.context)
    }

    @JavascriptInterface
    fun initiateFileSearch() {
        val chooseFile = Intent(Intent.ACTION_GET_CONTENT)
        chooseFile.addCategory(Intent.CATEGORY_OPENABLE)
        chooseFile.type = "*/*"
        startActivityForResult(
            context as MainActivity,
            Intent.createChooser(chooseFile, "Choose a file"),
            1,
            null
        )
    }

    @JavascriptInterface
    fun restartActivity() {
        val i = Intent(context, MainActivity::class.java)
        i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
        startActivity(context, i, null)
    }

    @JavascriptInterface
    fun registerDevice(message: String): String? {
        return connection.register(message)
    }

    @JavascriptInterface
    fun startAlarmAudio() {
        if (mediaPlayer.isLooping) {
            mediaPlayer.reset()
        }

        val audioManager = context.getSystemService(Context.AUDIO_SERVICE) as AudioManager

        if (this.getSetting("quiet_mode", "0").equals("1")) {
            audioManager.setStreamVolume(AudioManager.STREAM_MUSIC, 0, 0)
        } else {
            // CRANK THAT VOLUME
            audioManager.setStreamVolume(
                AudioManager.STREAM_MUSIC,
                audioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC),
                0
            )
        }

        val descriptor = context.assets.openFd("dist/audio/fire-alarm.mp3")
        mediaPlayer.setDataSource(descriptor.fileDescriptor, descriptor.startOffset, descriptor.length)
        descriptor.close()

        mediaPlayer.prepare()
        mediaPlayer.isLooping = true

        mediaPlayer.start()
    }

    @JavascriptInterface
    fun stopAlarmAudio() {
        mediaPlayer.stop()
    }

    @JavascriptInterface
    fun isAlarmAudioPlaying(): Boolean {
        return mediaPlayer.isPlaying
    }

    @JavascriptInterface
    fun silenceAlarmAudio() {
        val audioManager = context.getSystemService(Context.AUDIO_SERVICE) as AudioManager
        audioManager.setStreamVolume(AudioManager.STREAM_MUSIC, 0, 0)
    }

    @JavascriptInterface
    fun solvePanic(message: String): String? {
        return connection.solve(message)
    }

    @JavascriptInterface
    fun triggerPanic(message: String): String? {
        return connection.panic(message)
    }

    @JavascriptInterface
    fun getUpdates(): String? {
        return connection.update()
    }

    @JavascriptInterface
    fun downloadUpdate(url: String) {
        (context as MainActivity).downloadUpdate(url)
    }
}