package systems.kumi.expalert

import android.annotation.SuppressLint
import android.app.DownloadManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.Window
import android.view.WindowManager
import android.webkit.WebView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.FileProvider
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import java.io.BufferedReader
import java.io.File
import java.io.FileNotFoundException
import java.io.IOException
import java.io.InputStreamReader


class MainActivity : AppCompatActivity() {
    private val PICKFILE_REQUEST_CODE = 1
    var pendingIntent: Intent? = null

    val onDownloadComplete: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val id = intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, -1)
            Log.d("DOWNLOAD", "Downloaded file with id $id")
            launchInstaller(id, context, getSystemService(Context.DOWNLOAD_SERVICE) as DownloadManager)
        }
    }

    private val receiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            Log.d("PANIC", "Received panic broadcast")
            val status = intent.getBooleanExtra("panic", false)

            if (status && !this@MainActivity.hasWindowFocus()) {
                Log.d("PANIC", "Restarting activity")
                val intent = packageManager.getLaunchIntentForPackage(BuildConfig.APPLICATION_ID)
                intent!!.setPackage(null)
                intent!!.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                startActivity(intent)
            }

            val webView = findViewById<WebView>(R.id.webView)

            if (status) {
                val heartbeat = intent.getStringExtra("heartbeat")
                webView.evaluateJavascript("panicking('$heartbeat');", null)
            } else {
                webView.evaluateJavascript("notPanicking();", null)
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        LocalBroadcastManager.getInstance(this).unregisterReceiver(receiver)
        unregisterReceiver(onDownloadComplete)
    }

    override fun onStop() {
        super.onStop()

    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        setIntent(intent)
        pendingIntent = intent
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        requestWindowFeature(Window.FEATURE_NO_TITLE)
        window.setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN)
        window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)

        setContentView(R.layout.activity_main)

        val webView = findViewById<WebView>(R.id.webView)

        webView.addJavascriptInterface(AppFunctions(this), "AppFunctions")
        webView.settings.javaScriptEnabled = true

        webView.loadUrl("file:///android_asset/app.html")

        val serviceIntent = Intent(this, PollingService::class.java)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            startForegroundService(serviceIntent)
        } else {
            startService(serviceIntent)
        }

        LocalBroadcastManager.getInstance(this).registerReceiver(
            receiver,
            IntentFilter("systems.kumi.expalert.PANIC")
        )

        registerReceiver(onDownloadComplete, IntentFilter(DownloadManager.ACTION_DOWNLOAD_COMPLETE))

    }

    override fun onResume() {
        super.onResume()
    }

    override fun onPause() {
        super.onPause()
    }

    fun initiateFileSearch() {
        val chooseFile = Intent(Intent.ACTION_GET_CONTENT)
        chooseFile.addCategory(Intent.CATEGORY_OPENABLE)
        chooseFile.type = "*/*"
        startActivityForResult(
            Intent.createChooser(chooseFile, "Choose a file"),
            PICKFILE_REQUEST_CODE
        )
    }

    fun downloadUpdate(url: String) {
        var request = DownloadManager.Request(Uri.parse(url))
        request.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_WIFI or DownloadManager.Request.NETWORK_MOBILE)
        request.setTitle("Downloading EXPAlert Update")
        request.setDescription("Downloading...")

        request.allowScanningByMediaScanner()
        request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED)
        request.setDestinationInExternalFilesDir(this, null, "${System.currentTimeMillis()}.apk")

        var manager = getSystemService(Context.DOWNLOAD_SERVICE) as DownloadManager
        manager.enqueue(request)
    }

    class BootReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            if (intent.action.equals(Intent.ACTION_BOOT_COMPLETED, ignoreCase = true)) {
                val serviceIntent = Intent(context, MainActivity::class.java)
                context.startService(serviceIntent)
            }
        }
    }

    @SuppressLint("Range")
    private fun launchInstaller(downloadId: Long, context: Context, manager: DownloadManager) {
        val query = DownloadManager.Query().setFilterById(downloadId)
        val cursor = manager.query(query)

        Log.d("DOWNLOAD", "Processing file with id $downloadId")

        if (cursor.moveToFirst()) {
            val downloadStatus = cursor.getInt(cursor.getColumnIndex(DownloadManager.COLUMN_STATUS))
            if (DownloadManager.STATUS_SUCCESSFUL == downloadStatus) {
                val uriString = cursor.getString(cursor.getColumnIndex(DownloadManager.COLUMN_LOCAL_URI))
                val intent = Intent(Intent.ACTION_VIEW).apply {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                        setDataAndType(FileProvider.getUriForFile(context, context.packageName + ".provider", File(Uri.parse(uriString).path)), "application/vnd.android.package-archive")
                        flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_GRANT_READ_URI_PERMISSION
                    } else {
                        setDataAndType(Uri.parse(uriString), "application/vnd.android.package-archive")
                        flags = Intent.FLAG_ACTIVITY_NEW_TASK
                    }
                }
                context.startActivity(intent)
            }
        }
        cursor.close()
    }

    @Override
    protected override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == PICKFILE_REQUEST_CODE && resultCode == RESULT_OK) {
            val content_describer: Uri = data!!.data!!
            var reader: BufferedReader? = null
            try {
                // open the user-picked file for reading:
                val `in` = contentResolver.openInputStream(content_describer)
                // now read the content:
                reader = BufferedReader(InputStreamReader(`in`))
                var line: String?
                val builder = StringBuilder()
                while (reader.readLine().also { line = it } != null) {
                    builder.append(line)
                }

                val webView = findViewById<WebView>(R.id.webView)
                var text = builder.toString()

                webView.post(Runnable {
                    webView.evaluateJavascript("processJSON('$text');", null)
                    webView.evaluateJavascript("AppFunctions.restartActivity();", null)
                })

            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            } catch (e: IOException) {
                e.printStackTrace()
            } finally {
                if (reader != null) {
                    try {
                        reader.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }
                }
            }
        }
    }
}