package systems.kumi.expalert

import java.security.cert.CertificateException
import java.security.cert.X509Certificate
import javax.net.ssl.TrustManager
import javax.net.ssl.X509TrustManager

class MergeTrustManager(private val trustManagers: List<TrustManager>) : X509TrustManager {
    override fun checkClientTrusted(chain: Array<X509Certificate?>?, authType: String?) {
        trustManagers.forEach { trustManager ->
            if (trustManager is X509TrustManager) {
                try {
                    trustManager.checkClientTrusted(chain, authType)
                    return
                } catch (e: CertificateException) {
                    // Ignore as we want to try with other trust managers
                }
            }
        }
        throw CertificateException("None of the TrustManagers trust this client certificate")
    }

    override fun checkServerTrusted(chain: Array<X509Certificate?>?, authType: String?) {
        trustManagers.forEach { trustManager ->
            if (trustManager is X509TrustManager) {
                try {
                    trustManager.checkServerTrusted(chain, authType)
                    return
                } catch (e: CertificateException) {
                    // Ignore as we want to try with other trust managers
                }
            }
        }
        throw CertificateException("None of the TrustManagers trust this server certificate")
    }

    override fun getAcceptedIssuers(): Array<X509Certificate?> {
        val certificates = mutableListOf<X509Certificate?>()
        trustManagers.forEach { trustManager ->
            if (trustManager is X509TrustManager) {
                certificates.addAll(trustManager.acceptedIssuers)
            }
        }
        return certificates.toTypedArray()
    }
}
