package systems.kumi.expalert

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.Intent
import android.content.SharedPreferences
import android.os.Build
import android.os.IBinder
import android.preference.PreferenceManager
import android.util.Log

import androidx.core.app.NotificationCompat
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import java.util.Timer
import java.util.TimerTask

class PollingService : Service() {
    private val CHANNEL_ID = "EXPAlert"
    private val timer = Timer()
    var preferences: SharedPreferences? = null

    override fun onCreate() {
        super.onCreate()
        preferences = PreferenceManager.getDefaultSharedPreferences(this)
    }

    private fun getPollingInterval(): Long {
        val interval = this.getSetting("polling_interval", "1")
        return interval!!.toLong() * 1000
    }

    private fun getSetting(key: String, default: String = ""): String? {
        var value = preferences!!.getString(key, default)
        return if (value.isNullOrBlank()) default else value
    }

    override fun onBind(intent: Intent): IBinder? {
        return null
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        createNotificationChannel()

        Thread.setDefaultUncaughtExceptionHandler { _, e ->
            Log.e("TAG", "Timer thread exception", e)
        }

        val notification = NotificationCompat.Builder(this, CHANNEL_ID)
            .setContentTitle("EXPAlert Polling Service")
            .setContentText("Checking server response...")
            .setSmallIcon(R.drawable.ic_launcher_foreground)
            .build()

        startForeground(1, notification)

        timer.scheduleAtFixedRate(
            object : TimerTask() {
                override fun run() {
                    Log.d("HEARTBEAT", "Polling service running")
                    val connection = ServerConnection(this@PollingService)

                    try {
                        Log.d("HEARTBEAT", "Polling service checking heartbeat")
                        val heartbeat = connection.heartbeat()

                        Log.d("HEARTBEAT", "Polling service heartbeat: $heartbeat")

                        if (heartbeat != null && heartbeat.contains("\"panic\":")) {
                            val status = heartbeat.contains("\"panic\": true")
                            Log.d("HEARTBEAT", "Polling service status: $status")
                            val intent = Intent("systems.kumi.expalert.PANIC")
                            intent.putExtra("panic", status)
                            intent.putExtra("heartbeat", heartbeat)
                            LocalBroadcastManager.getInstance(this@PollingService).sendBroadcast(intent)
                        }
                    } catch (e: Exception) {
                        Log.e("HEARTBEAT", "Polling service exception", e)
                    }
                }
            },
            0,
            this.getPollingInterval()
        )

        return START_REDELIVER_INTENT
    }

    override fun onDestroy() {
        timer.cancel()
        super.onDestroy()
    }

    private fun createNotificationChannel() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val serviceChannel = NotificationChannel(
                CHANNEL_ID,
                "Foreground Service Channel",
                NotificationManager.IMPORTANCE_HIGH
            )
            val manager = getSystemService(NotificationManager::class.java)
            manager.createNotificationChannel(serviceChannel)
        }
    }
}
